<?php
#
# file: install.inc
#
# Zend installation scripts library - general routines
#  Copyright 2001 Zend Technologies Ltd. All Rights Reserved.
#


error_reporting (E_ALL);
define ('TMPDIR', $TMPDIR);

###################### global variables ###############################
$PREFIX = '/usr/local/Zend';
#######################################################################

require_once ('install-text.inc');
require_once ('phpini.inc');
require_once ('get_license_ui.inc'); 



###################### functions ######################################
# return - last line of command output
function do_command ($command, $errmsg = '')
{
	$out = exec ($command, $a, $rc);
	if ($rc != 0)
	{
		on_error ($errmsg == '' ? $errmsg : "Error executing $command");
	}
	return ($out);
} # end of do_command()



# arguments: 0 - command to execute
#            1 - error message
# returns: file descriptor (the same as popen())
function popen_wrap ($command, $errmsg = '')
{
	$fd = popen ($command, 'w');
	if (! $fd)
	{
		on_error ($errmsg == '' ? "Error executing $command" : $errmsg);
	}
	return ($fd);
} # end of popen_wrap()



function rename_wrap ($from, $to, $errmsg = '')
{
	# if (! rename ($from, $to))
	# ugly hack instead of php rename, which fails on moving
	# files accross file-systems
        unlink_silent ($to);
	do_command ("mv $from $to", ($errmsg == '' ? "Error moving $from to $to" : $errmsg));
	/*
	if (!file_exists($to)) 
	{
		on_error ($errmsg == '' ? "Error moving $from to $to" : $errmsg);
	}
	*/
} # end of rename_wrap()


function copy_wrap ($from, $to)
{
        unlink_silent ($to);
	copy ($from, $to) || on_error ("Error copying $from to $to");
	chmod ($to, 0644);
}


function fopen_wrap ($file, $perms)
{
	$fd = fopen ($file, $perms);
	if (! $fd)
	{
		on_error ("Error opening $file");
	}
	return ($fd);
}



function fwrite_wrap ($file, $perms, $str)
{
	$fd = fopen_wrap ($file, $perms);
	fwrite ($fd, $str) || on_error ("Error writing $file");
	fclose ($fd)       || on_error ("Error writing $file");
}




# first argument - function name,
# subsequrnt arguments (if any) - function arguments
function run_silently ($function)
{
	$args = func_get_args();
	array_shift ($args);
	error_reporting (E_ALL & ~E_WARNING & ~E_NOTICE);
	eval ('$r = $function (\'' . join ('\', \'', $args) . '\');');
	error_reporting (E_ALL);
	return ($r);
} # end of run_silently()

function is_dir_silent ($dir)
{
	return (run_silently ('is_dir', $dir));
}
function is_file_silent ($file)
{
	return (run_silently ('is_file', $file));
}
function is_link_silent ($file)
{
        return (run_silently ('is_link', $file));
}
function mkdir_silent ($dir)
{
	run_silently ('mkdir', $dir, 0755);
}
function unlink_silent ($file)
{
        run_silently ('unlink', $file);
}


function symlink_silent($target,$link) {
	if (file_exists($link)) {
		if (is_link($link) and readlink($link)===$target) {
			// Already exists
			return true;
		}
		return false;
	}
	return (run_silently ('symlink', $target,$link));
}



# read a file into a string
function file2str ($file)
{
	if (is_dir_silent($file)) {
		on_error('Can\'t read "'.$file.'": It should not be a directory.');
	}
	$fd = fopen_wrap ($file, 'r');
	$str = '';
	while (! feof ($fd))
	{
		$str .= fread ($fd, 4096);
	}
	return ($str);
} # end of file2str()



# compares two files
# return: -1, 0, or 1, as in strcmp()
function filecmp ($file0, $file1)
{
	return (strcmp (file2str ($file0), file2str ($file1)));
} # end of filecmp()



# prints message and returns user input
# (used only by tty)
function readline ($msg = 'To continue, press Enter')
{
	print ($msg);
	flush ();
	exec ("read OUTPUT\necho \$OUTPUT", $output);
	return (isset ($output[0]) ? $output[0] : '');
} # end of readline()




function cleanup ()
{
	if (is_dir_silent (TMPDIR))
	{
		if (file_exists (TMPDIR . '/ZEND_INSTALL_TMPDIR'))
		{
			exec ('rm -rf ' . TMPDIR);
		}
		else
		{
			print ('WARNING: Installation temporary directory ' . TMPDIR . "\n");
			print ("has been tampered. Not removing it.\n");
		}
	}
} # end of cleanup()



function on_error ($msg)
{
	cleanup();
	print ("\n$msg\n");
	die ();
} # end of on_error()



# argument - message to display
function on_abort ($msg = '')
{
	cleanup();
	system ('clear');
	print (PRODUCT . " installation script was aborted. The process was not completed\n");
	print ("successfully.\n$msg\n");
	die ();
} # end of on_abort()



# arguments: 0 - dialog box option; one of: msgbox,
#            1 - message to display
#            2, ... - any other option to dialog (if any)
# return - stderr of dialog output; in case of 'yesno' box, return 1 for 'yes' and 0 for 'no'
function rundialog ($mode, $msg)
{
	$command = DIALOG;
	switch ($mode)
	{
	case 'textbox':
		$command .= " --$mode \"$msg\" 0 0";
		break;
	case 'msgbox':
	case 'inputbox':
	case 'passwordbox':
	case 'yesno':
	case 'infobox':
	case 'menu':
		if ($mode=='infobox') {
			$command .= ' --sleep 1';
		}
		# remove ascii escape sequences (not suitable for dialog)
		$msg = preg_replace ('/\\033[0-9;\[]*m/', '', $msg);
		$command .= " --$mode \"$msg\n\" 0 0";
		break;
	default:
		print ("Unsupported dialog option\n");
		return ('');
	}
	
	$args = func_get_args();
	array_shift ($args);
	array_shift ($args);
	if (sizeof ($args) != 0)
	{
		$command .= ' "' . join (', "', $args) . '"';
	}
	$TMPFILE = TMPDIR . '/dialog';
	$fd = popen_wrap ("$command 2>$TMPFILE", 'Error executing dialog utility');
	$rc = pclose ($fd);
	if (($mode == 'inputbox')&&($rc>0)) {
		// Cancel pressed.
		return false;
	}
	return ($mode == 'yesno' ? 1 - ($rc >> 8) : file2str ($TMPFILE));
} # end of rundialog()



# argument - message to display
function msgbox ($msg)
{
	if (defined ('DIALOG'))
	{
		rundialog ('msgbox', "\n$msg");
	}
	else
	{
		print ("\n$msg\n");
		readline();
	}
} # end of msgbox()



# args: 1 - message to display
#       2 - initial variable value
# return: value of inputbox
function inputbox ($msg, $initval)
{
	do {
		if (defined ('DIALOG')) {
			$input = rundialog ('inputbox', $msg, $initval);
			if ($input===false) {
				on_abort();
			}
		}
		else
		{
			$input = readline ("$msg [$initval]: ");
			if ($input == '')
			{
				$input = $initval;
			}
  		}
	} while (empty($input) || (ereg("^ *$", $input)));
  return ($input);
} # end of inputbox()



# argument: message to display
function infobox ($msg)
{
	if (defined ('DIALOG'))
	{
		rundialog ('infobox', $msg);
	}
	else
	{
		print ("$msg\n");
	}
} # end of infobox()



# args: message to display
# return: 0 or 1
function yesnobox ($msg)
{
	if (defined ('DIALOG'))
	{
		return (rundialog ('yesno', $msg));
	}
	else
	{
		print ("\n$msg\n");
		while (TRUE)
		{
			$output = readline ("Answer (yes or no): ");
			if (preg_match ('/^y/i', $output))
			{
				return (1);
			}
			elseif (preg_match ('/^n/i', $output))
			{
				return (0);
			}
		}
	}
} # end of yesnobox()



# return: choosen password
function passwdbox ()
{
	while (1)
	{
		$msg = "\nEnter password for Zend Products User Interface.\nThe password should contain at least 4 characters:";
		if (defined ('DIALOG'))
		{
			$passwd = rundialog ('passwordbox', $msg);
		}
		else
		{
			print ($msg);
			exec ('stty -echo');
			$passwd = readline('');
		}
		if (strlen ($passwd) < 4)
		{
			msgbox ('A password should contain at least 4 characters. Please try again.');
			continue;
		}
		$msg="\nVerify the password: ";
		if (defined ('DIALOG'))
		{
			$checkpasswd = rundialog ('passwordbox', $msg);
		}
		else
		{
			print ("$msg");
			exec ('stty -echo');
			$checkpasswd = readline('');
			exec ('stty echo');
			print ("\n\n");
		}
		if (strcmp ($passwd, $checkpasswd) == 0)
		{
			break;
		}
		msgbox ('Passwords did not match, please try again.');
	}
	return ($passwd);
} # end of passwd()



function menubox($msg,$menuarr)
{
	if (defined ('DIALOG'))
	{
		$menuparams='';
		foreach ($menuarr as $tag=>$menuitem) {
			$menuparams.=' "'.$tag.'" "'.$menuitem.'"';
		}
		return (rundialog ('menu', $msg,'"'.count($menuarr).' '.$menuparams.'"'));
	}
	else
	{
		print ("\n$msg\n");
		foreach ($menuarr as $tag=>$menuitem) {
			print "$tag) $menuitem\n";
		}
		while (TRUE)
		{
			$output = readline ("Please select: ");
			if (!empty($menuarr[$output])) {
				return $output;
			}
		}
	}
} # end of menubox()

function select_directory($msg, $default, $is_existing_directory) {
	while (1) {
		$dir = inputbox ($msg, $default);
		$i=strlen($dir);
		if ($i>0 and substr($dir,$i-1,1)==='/') {
			// Remove trailing '/'
			$apache_install_dir=substr($dir,0,$i-1);
		}
		if ($is_existing_directory) {
			if (!is_dir_silent($dir)) {
				msgbox('"'.$dir.'" is not a directory.');
			} else {
				break;
			}
		} else {
			if (!is_dir_silent(dirname($dir))) {
				msgbox('"'.dirname($dir).'" is not a directory.');
			} else if (file_exists ($dir)) {
				msgbox('"'.$dir.'" already exists and will not be override.');
			} else {
				break;
			}
		}
	}
	return $dir;
}


function choose_prefix ()
{
	global $PREFIX;
	$PREFIX = inputbox ('Specify the location where to install ' . PRODUCT, $PREFIX);
	system ("mkdir -p $PREFIX");
	if (! is_dir_silent ($PREFIX))
	{
		on_error ("Directory $PREFIX not exists and unable to create it");
	}
} # end of choose_prefix()



# return Apache's Docroot
function guess_apache_docroot ()
{
	$APACHE_DOCROOT = '';
	# install-from-src redhat slackware suse
	$locations = array (0=>'/usr/local/apache/htdocs', 1=>'/var/www/html', 2=>'/var/lib/apache/htdocs', 3=>'/usr/local/httpd/htdoc');
	$APACHE_DOCROOT=$locations[0];
	foreach ($locations as $location)
	{
		if (is_dir_silent ($location))
		{
			$APACHE_DOCROOT = $location;
			break;
		}
	}
	while (1) {
		$rc = inputbox ("Specify the location of your Apache document root directory", $APACHE_DOCROOT);
		if ($rc==false) {
			on_abort();
		}
		$APACHE_DOCROOT=$rc;
		if (! is_dir_silent ($APACHE_DOCROOT))
		{
			msgbox("Directory $APACHE_DOCROOT does not exist.");
		} else {
			break;
		}
	}
	return ($APACHE_DOCROOT);
} # end of guess_apache_docroot()





# install directory/file to given place, with backup
# args: 1 - what to install
#       2 - where to install
function do_install ($WHAT, $WHERE)
{
        global $PHP_INI;

	infobox ("Installing $WHAT to $WHERE");
	
	# Installing lmutil For FreeBSD 4.0 and above: check for libc.so.3 in /usr/lib
	# (if compat libs are not installed there will probably be no libc.so.3)
	# if none check for libc.so or libc.so.4, if that exists link 3 to it
	
	if (($WHAT == "lmutil") && (`uname -s` == "FreeBSD")) {
		if (!file_exists("/usr/lib/libc.so.3")) {
			if (file_exists("/usr/lib/libc.so"))
			symlink("/usr/lib/libc.so", "/usr/lib/libc.so.3");
			elseif (file_exists("/usr/lib/libc.so.4"))
			symlink("/usr/lib/libc.so.4", "/usr/lib/libc.so.3");
		}
	}
	
	$CURDIR = getcwd();
	if (empty ($CURDIR))
	{
		on_error ("Unable get current working directory");
	}
	if (file_exists ("../data/$WHAT"))
	{
		chdir ('../data') || on_error ("Unable chdir to data directory");
	}
	elseif (file_exists ("../$WHAT"))
	{
		chdir ('..') || on_error ("Unable chdir to the main installation directory");
	}
	elseif (file_exists ("../install/$WHAT"))
	{
		chdir ('../install') || on_error ("Unable chdir to install directory");
	}
	else
	{
		on_error ("$WHAT not found");
	}
	
	mkdir_silent ($WHERE);
	if (! is_dir_silent ($WHERE))
	{
		on_error ("Error creating $WHERE");
	}
	
	# if it's a bin or sbin dir the need to make sure
	# persmissions are correct:
	# $pp = pathinfo("$WHERE");
	# if (($pp["basename"] == "bin") || ($pp["basename"] == "sbin"))
	# chmod ($WHERE, 0755);
	
	
	if (is_file_silent ($WHAT))
	{
	  $dest = "$WHERE/$WHAT";
	  if ($dest == $PHP_INI)
	    {
	      backup_php_ini();
	    }
	  else
	    {
		# backup a single file
	        $backup = "$dest." . BACKUP_SUFFIX;
		if ((is_file_silent ($dest) && (filecmp ($WHAT, $dest) != 0)) || is_link_silent ($dest))
		{
		  rename_wrap ($dest, $backup, "Error saving previous copy of $WHAT");
		}
	    }
		# do install itself
		do_command ("cp -p $WHAT $WHERE/", "Error installing $WHAT");
	}
	elseif (is_dir_silent ($WHAT))
	{
		# backup a directory
		chdir ($WHAT) || on_error ("Unable chdir to $WHAT");
		$command = "find . -type f -exec test -f $WHERE/{} \; ! -exec cmp -s {} $WHERE/{} \; -exec mv $WHERE/{} $WHERE/{}." . BACKUP_SUFFIX . ' \;';
		do_command ($command, "Error backuping $WHAT");
		
		# do install itself
		do_command ("cp -pr * $WHERE/", "Error installing $WHAT");
	}
	else
	{
		on_error ("$WHAT no found");
	}
	chdir ($CURDIR) || on_error ("Unable restore the current working directory");
} # end of do_install()





# append/create a license file
# arg: basename of a new license file
function install_license ($file)
{
	global $PREFIX,$LICENSE_MISSING;
	
	$new_file = "../data/$file";
	$old_file = "$PREFIX/$file";
	
	if (file_exists ($new_file))
	{
		infobox ("Installing license $file");
		if (is_file_silent ($old_file))
		{
			if (filecmp ($new_file, $old_file) != 0)
			{
				fwrite_wrap ($old_file, 'a', file2str ($new_file));
			}
		}
		else
		{
			copy_wrap ($new_file, $old_file);
		}
		return true;
	}
	else
	{
		$LICENSE_MISSING=true;
		return false;
	}
} # end of install_license()




# args: 1 - url of product UI (if empty, do not display corresponding message at all)
#       2 - additonal message
function byebye($url = '', $msg = '',$need_web_server_restart=true)
{
	global $HOSTNAME,$LICENSE_MISSING;
	system('clear');
	print "\n\n\n\n\n\n\n";
	print "******************************************************************************\n";
	print "\n";
	if ($LICENSE_MISSING) {
		print "  [7;1mYou must [0m[37;44;1mINSTALL the license[0m[7;1m files from Zend.com to your installation directory[0m\n";
		print "\n";
	}
	if ($need_web_server_restart) {
		print "  [7;1mYou must [0m[37;44;1mRESTART your Web server[0m[7;1m for the modifications to take effect[0m\n";
		print "\n";
	}
	if (!empty($url)) {
		$url_with_portnumber=preg_replace("@://([^/])/@", "://\\1:<port-number>/", $url);
		print "  You can access the ".PRODUCT."'s User Interface using the URL:\n";
		print "  $url\n";
		print "\n";
		print " (If your webserver is running on port different from the default\n";
		print "  port 80, you should access the ".PRODUCT." as\n";
		print "  $url_with_portnumber\n";
		print "\n";
	}
	if (!empty($msg)) {
		print $msg;
	}
	print "\n";
	print "******************************************************************************\n\n\n\n\n\n";
} # end of byebye()




#######################################################################
#
# Add zendtech group to /etc/group if it doesn't yet exist
#
#######################################################################
function do_add_group()
{
	$group_exists = exec("cut -d: -f1 /etc/group | grep zendtech");
	if ($group_exists) {
		return;
	}
	
	$gid = 2000;
	while (true) {
		$gid_exists = exec("cut -d: -f3 /etc/group | grep $gid");
		if (!$gid_exists) {
			break;
		}
		$gid++;
	}
	
	$f = fopen("/etc/group", "a");
	if (!$f) {
		on_error("Unable to add zendtech group");
	}
	fwrite($f, "zendtech:x:$gid:\n");
	fclose($f);
}




#######################################################################
#
# Read permissions file and update files
# after installation.
#
#######################################################################
function do_file_permissions()
{
	global $PREFIX, $PHP_INI;
	$line = 0;
	
		do_add_group();
	do_command("chgrp zendtech $PHP_INI", "Can't set group for php.ini");
	do_command("chmod g+rw $PHP_INI", "Can't set permissions for php.ini");
	
	if (file_exists ('permissions'))
	{
		if (!($fd=fopen("permissions", "r")))
		{
			on_error ("Can't open permissions file");
		}
		
		while (($buf=fgets($fd, 1024)))
		{
			# skip lines that start with # or \n
			if (preg_match('/^[#\n]/', $buf))
			continue;
			
			$perms = sscanf ($buf, "%s\t%s\t%s\t%s");
			list ($fname, $mode, $owner, $group) = $perms;
			
			if ($owner && $owner!=-1) {
				if (!chown("$PREFIX/$fname", $owner)) {
					on_error ("Can't set ownership for $PREFIX/$fname");
				}
			}
			if ($group && $group!=-1) {
				if (!chgrp("$PREFIX/$fname", $group)) {
					on_error ("Can't set group for $PREFIX/$fname");
				}
			}
			if (!chmod("$PREFIX/$fname", octdec($mode))) {
				on_error ("Can't set permissions for $PREFIX/$fname");
			}
		}
		
		fclose ($fd);
	}
} # end of do_file_permissions()


#######################################################################
#
# A set of functions dealing with transfering information from package BUILD file
# to php.ini as directives.
#
#######################################################################

function ini_array_to_ini_modifier_commands($ini_arr) {
	if (!is_array($ini_arr)) return '';
	$result='';
	foreach ($ini_arr as $directive=>$value) {
		$result.='modify entry Zend '.$directive.' "'.$value.'"'."\n";
	}
	return $result;
}

function rebuild_ini_array($ini_arr,$translation_arr) {
	if (!is_array($ini_arr) or !is_array($translation_arr)) return array();
	if (empty($ini_arr) or empty($translation_arr)) return array();
	$result=array();
	foreach ($ini_arr as $directive=>$value) {
		foreach ($translation_arr as $prefix=>$translated_prefix) {
			$prefix_length=strlen($prefix);
			if (strncmp($prefix,$directive,$prefix_length)==0) {
				$translated_directive=$translated_prefix.substr($directive,$prefix_length);
				$result[$translated_directive]=$value;
			}
		}
	}
	return $result;
}

function get_build_info() {
	if (!file_exists('../BUILD')) return array();
	return parse_ini_file('../BUILD');
}


###################### end of functions ###############################




###################### initial actions ################################
$LICENSE_MISSING=false;
define ('BASENAME', basename ($CALLING_SCRIPT));
if (BASENAME == 'install.sh')
{
	define ('DIALOG', './dialog --title "' . PRODUCT . ' ' . VERSION . '"');
}
elseif (BASENAME != 'install-tty.sh')
{
	on_error ('Script called under a wrong name: ' . BASENAME);
}

if (posix_getuid() != 0)
{
	on_error ('You need root privileges to run this script');
}

mkdir (TMPDIR, 0700) || on_error ('Unable create temporary directory ' . TMPDIR);
touch (TMPDIR . '/ZEND_INSTALL_TMPDIR') || on_error ('Unable create file in temporary directory');

$uname = posix_uname();
$OS = $uname['sysname'];
$HOSTNAME = $uname['nodename'];
if ($OS == '')
{
	on_error ("Unable to determine OS");
}
$GROUP = ($OS == 'FreeBSD' ? 'wheel' : 'root');


$msg  = 'Welcome to the ' . PRODUCT . ' ' . VERSION . " Installation Script!\n";
$msg .= "For more information on this script, see the Installation\n";
$msg .= 'section in the ' . PRODUCT . " User Manual.";
msgbox ($msg);

if (file_exists('../LICENSE')) {
	// If specific license file exists, it is displayed
	if (file_exists('../LICENSE')) {
		$rc = yesnobox("\nDo you want to view the license agreement?");
		if ($rc) {
			if (defined ('DIALOG')) {
				rundialog('textbox','../LICENSE');
			} else {
				system ('clear');
				popen("more ../LICENSE","w");
				system ('clear');
			}
		}
	}
}
//READ THESE TERMS CAREFULLY BEFORE DOWNLOADING OR INSTALLING THIS SOFTWARE.
$LICENSE = <<<EOC

IMPORTANT: 
BY SELECTING THE "YES" OPTION BELOW, DOWNLOADING, INSTALLING, OR
OTHERWISE USING THIS SOFTWARE, YOU ACKNOWLEDGE THAT YOU HAVE READ THE
LICENSE AGREEMENT, AND THAT YOU AGREE TO BE BOUND BY ITS TERMS AND
CONDITIONS.
IF YOU DO NOT AGREE TO ALL OF THE TERMS AND CONDITIONS OF SUCH AGREEMENT,
YOU ARE NOT AN AUTHORIZED USER OF THE SOFTWARE AND IT IS YOUR
RESPONSIBILITY TO EXIT THIS DOWNLOADING/INSTALLATION PROCESS WITHOUT
DOWNLOADING OR INSTALLING THE SOFTWARE BY SELECTING THE "NO" OPTION BELOW,
AND TO DELETE THE SOFTWARE FROM YOUR COMPUTER.
EOC;


$rc = yesnobox($LICENSE."\n\nDo you accept the terms of the license?");
if (!$rc)
{
	cleanup();
	exit();
}

/*
if (defined ('DIALOG'))
{
}
else
{
	system ('clear');
	$LICENSE = preg_replace ('/SELECTING THE "YES" BUTTON/', 'PRESSING "ENTER"', $LICENSE);
	$LICENSE = preg_replace ('/SELECTING THE "NO" BUTTON/', 'PRESSING Ctrl-C', $LICENSE);
	print ("$LICENSE\n\n");
	readline ('Press Enter if you accept the terms of the license and Ctrl-C if you don\'t');
	system ('clear');
}
*/

###################### end of initial actions #########################


# end of install.inc
?>
